@@ -5,7 +5,6 @@ from typing import (
     TYPE_CHECKING,
     Any,
     Callable,
-    Iterable,
     List,
     Optional,
     Sequence,
@@ -16,15 +15,13 @@ from typing import (
 )
 from uuid import UUID, uuid4
 
+from sqlfluff.core.helpers.string import curtail_string
 from sqlfluff.core.parser.context import ParseContext
-from sqlfluff.core.parser.helpers import trim_non_code_segments
-from sqlfluff.core.parser.match_logging import parse_match_logging
+from sqlfluff.core.parser.match_algorithms import greedy_match
 from sqlfluff.core.parser.match_result import MatchResult
-from sqlfluff.core.parser.match_wrapper import match_wrapper
 from sqlfluff.core.parser.matchable import Matchable
-from sqlfluff.core.parser.segments import BaseSegment, allow_ephemeral
-from sqlfluff.core.parser.types import MatchableType, SimpleHintType
-from sqlfluff.core.string_helpers import curtail_string
+from sqlfluff.core.parser.segments import BaseSegment
+from sqlfluff.core.parser.types import ParseMode, SimpleHintType
 
 if TYPE_CHECKING:  # pragma: no cover
     from sqlfluff.core.dialects.base import Dialect
@@ -84,15 +81,18 @@ class BaseGrammar(Matchable):
 
     is_meta = False
     equality_kwargs: Tuple[str, ...] = ("_elements", "optional", "allow_gaps")
+    # All grammars are assumed to support STRICT mode by default.
+    # If they wish to support other modes, they should declare
+    # it by overriding this attribute.
+    supported_parse_modes: Set[ParseMode] = {ParseMode.STRICT}
 
     @staticmethod
-    def _resolve_ref(elem: Union[str, MatchableType]) -> MatchableType:
+    def _resolve_ref(elem: Union[str, Matchable]) -> Matchable:
         """Resolve potential string references to things we can match against."""
         if isinstance(elem, str):
             return Ref.keyword(elem)
         elif isinstance(elem, Matchable):
-            return elem
-        elif issubclass(elem, BaseSegment):
+            # NOTE: BaseSegment types are an instance of Matchable.
             return elem
 
         raise TypeError(
@@ -102,12 +102,12 @@ class BaseGrammar(Matchable):
 
     def __init__(
         self,
-        *args: Union[MatchableType, str],
+        *args: Union[Matchable, str],
         allow_gaps: bool = True,
         optional: bool = False,
-        ephemeral_name: Optional[str] = None,
-        terminators: Sequence[Union[MatchableType, str]] = (),
+        terminators: Sequence[Union[Matchable, str]] = (),
         reset_terminators: bool = False,
+        parse_mode: ParseMode = ParseMode.STRICT,
     ) -> None:
         """Deal with kwargs common to all grammars.
 
@@ -124,13 +124,6 @@ class BaseGrammar(Matchable):
                 is this grammar *optional*, i.e. can it be skipped if no
                 match is found. Outside of a Sequence, this option does nothing.
                 Defaults `False`.
-            ephemeral_name (:obj:`str`, optional): If specified this allows
-                the grammar to match anything, and create an EphemeralSegment
-                with the given name in its place. The content of this grammar
-                is passed to the segment, and will become the parse grammar
-                for it. If used widely this is an excellent way of breaking
-                up the parse process and also signposting the name of a given
-                chunk of code that might be parsed separately.
             terminators (Sequence of :obj:`str` or Matchable): Matchable objects
                 which can terminate the grammar early. These are also used in some
                 parse modes to dictate how many segments to claim when handling
@@ -143,11 +136,17 @@ class BaseGrammar(Matchable):
                 before matching child elements. Situations where this might be
                 appropriate are within bracketed expressions, where outer
                 terminators should be temporarily ignored.
+            parse_mode (:obj:`ParseMode`): Defines how eager the grammar should
+                be in claiming unmatched segments. By default, grammars usually
+                only claim what they can match, but by setting this to something
+                more eager, grammars can control how unparsable sections are
+                treated to give the user more granular feedback on what can (and
+                what *cannot*) be parsed.
         """
         # We provide a common interface for any grammar that allows positional elements.
         # If *any* for the elements are a string and not a grammar, then this is a
         # shortcut to the Ref.keyword grammar by default.
-        self._elements: List[MatchableType] = [self._resolve_ref(e) for e in args]
+        self._elements: List[Matchable] = [self._resolve_ref(e) for e in args]
 
         # Now we deal with the standard kwargs
         self.allow_gaps = allow_gaps
@@ -156,17 +155,16 @@ class BaseGrammar(Matchable):
         # The intent here is that if we match something, and then the _next_
         # item is one of these, we can safely conclude it's a "total" match.
         # In those cases, we return early without considering more options.
-        self.terminators: Sequence[MatchableType] = [
+        self.terminators: Sequence[Matchable] = [
             self._resolve_ref(t) for t in terminators
         ]
         self.reset_terminators = reset_terminators
 
-        # ephemeral_name is a flag to indicate whether we need to make an
-        # EphemeralSegment class. This is effectively syntactic sugar
-        # to allow us to avoid specifying a EphemeralSegment directly in a dialect.
-        # If this is the case, the actual segment construction happens in the
-        # match_wrapper.
-        self.ephemeral_name = ephemeral_name
+        assert parse_mode in self.supported_parse_modes, (
+            f"{self.__class__.__name__} does not support {parse_mode} "
+            f"(only {self.supported_parse_modes})"
+        )
+        self.parse_mode = parse_mode
         # Generate a cache key
         self._cache_key = uuid4().hex
 
@@ -184,21 +182,6 @@ class BaseGrammar(Matchable):
         """
         return self.optional
 
-    @match_wrapper()
-    @allow_ephemeral
-    def match(
-        self, segments: Tuple[BaseSegment, ...], parse_context: ParseContext
-    ) -> MatchResult:
-        """Match a list of segments against this segment.
-
-        Matching can be done from either the raw or the segments.
-        This raw function can be overridden, or a grammar defined
-        on the underlying class.
-        """
-        raise NotImplementedError(
-            f"{self.__class__.__name__} has no match function implemented"
-        )  # pragma: no cover
-
     @cached_method_for_parse_context
     def simple(
         self, parse_context: ParseContext, crumbs: Optional[Tuple[str]] = None
@@ -206,275 +189,12 @@ class BaseGrammar(Matchable):
         """Does this matcher support a lowercase hash matching route?"""
         return None
 
-    @staticmethod
-    def _first_non_whitespace(
-        segments: Iterable["BaseSegment"],
-    ) -> Optional[Tuple[str, Set[str]]]:
-        """Return the upper first non-whitespace segment in the iterable."""
-        for segment in segments:
-            if segment.first_non_whitespace_segment_raw_upper:
-                return (
-                    segment.first_non_whitespace_segment_raw_upper,
-                    segment.class_types,
-                )
-        return None
-
-    @classmethod
-    def _prune_options(
-        cls,
-        options: List[MatchableType],
-        segments: Tuple[BaseSegment, ...],
-        parse_context: ParseContext,
-    ) -> List[MatchableType]:
-        """Use the simple matchers to prune which options to match on.
-
-        Works in the context of a grammar making choices between options
-        such as AnyOf or the content of Delimited.
-        """
-        available_options = []
-        prune_buff = []
-
-        # Find the first code element to match against.
-        first_segment = cls._first_non_whitespace(segments)
-        # If we don't have an appropriate option to match against,
-        # then we should just return immediately. Nothing will match.
-        if not first_segment:
-            return options
-        first_raw, first_types = first_segment
-
-        for opt in options:
-            simple = opt.simple(parse_context=parse_context)
-            if simple is None:
-                # This element is not simple, we have to do a
-                # full match with it...
-                available_options.append(opt)
-                continue
-
-            # Otherwise we have a simple option, so let's use
-            # it for pruning.
-            simple_raws, simple_types = simple
-            matched = False
-
-            # We want to know if the first meaningful element of the str_buff
-            # matches the option, based on either simple _raw_ matching or
-            # simple _type_ matching.
-
-            # Match Raws
-            if simple_raws and first_raw in simple_raws:
-                # If we get here, it's matched the FIRST element of the string buffer.
-                available_options.append(opt)
-                matched = True
-
-            # Match Types
-            if simple_types and not matched and first_types.intersection(simple_types):
-                # If we get here, it's matched the FIRST element of the string buffer.
-                available_options.append(opt)
-                matched = True
-
-            if not matched:
-                # Ditch this option, the simple match has failed
-                prune_buff.append(opt)
-                continue
-
-        return available_options
-
-    @classmethod
-    def _longest_trimmed_match(
-        cls,
-        segments: Tuple[BaseSegment, ...],
-        matchers: List[MatchableType],
-        parse_context: ParseContext,
-        trim_noncode: bool = True,
-    ) -> Tuple[MatchResult, Optional[MatchableType]]:
-        """Return longest match from a selection of matchers.
-
-        Prioritise the first match, and if multiple match at the same point the longest.
-        If two matches of the same length match at the same time, then it's the first in
-        the iterable of matchers.
-
-        Returns:
-            `tuple` of (match_object, matcher).
-
-        NOTE: This matching method is the workhorse of the parser. It drives the
-        functionality of the AnyOf & AnyNumberOf grammars, and therefore by extension
-        the degree of branching within the parser. It's performance can be monitored
-        using the `parse_stats` object on the context.
-
-        The things which determine the performance of this method are:
-        1. Pruning. This method uses `_prune_options()` to filter down which matchable
-           options proceed to the full matching step. Ideally only very few do and this
-           can handle the majority of the filtering.
-        2. Caching. This method uses the parse cache (`check_parse_cache` and
-           `put_parse_cache`) on the ParseContext to speed up repetitive matching
-           operations. As we make progress through a file there will often not be a
-           cached value already available, and so this cache has the greatest impact
-           within poorly optimised (or highly nested) expressions.
-        3. Terminators. By default, _all_ the options are evaluated, and then the
-           longest (the `best`) is returned. The exception to this is when the match
-           is `complete` (i.e. it matches _all_ the remaining segments), or when a
-           match is followed by a valid terminator (i.e. a segment which indicates
-           that the match is _effectively_ complete). In these latter scenarios, the
-           _first_ complete or terminated match is returned. In the ideal case, the
-           only matcher which is evaluated should be the "correct" one, and then no
-           others should be attempted.
-        """
-        # Have we been passed an empty list?
-        if len(segments) == 0:  # pragma: no cover
-            return MatchResult.from_empty(), None
-        # If presented with no options, return no match
-        elif not matchers:
-            return MatchResult.from_unmatched(segments), None
-
-        # Prune available options, based on their simple representation for efficiency.
-        available_options = cls._prune_options(
-            matchers, segments, parse_context=parse_context
-        )
-
-        # If we've pruned all the options, return no match
-        if not available_options:
-            return MatchResult.from_unmatched(segments), None
-
-        terminated = False
-
-        parse_context.increment("ltm_calls")
-        # NOTE: The use of terminators is only available via the context.
-        # They are set in that way to allow appropriate inheritance rather
-        # than only being used in a per-grammar basis.
-        if parse_context.terminators:
-            parse_context.increment("ltm_calls_w_ctx_terms")
-            terminators = parse_context.terminators
-        else:
-            terminators = ()
-
-        # If gaps are allowed, trim the ends.
-        if trim_noncode:
-            pre_nc, segments, post_nc = trim_non_code_segments(segments)
-
-        # At parse time we should be able to count on there being a location.
-        assert segments[0].pos_marker
-
-        # Characterise this location.
-        # Initial segment raw, loc, type and length of segment series.
-        loc_key = (
-            segments[0].raw,
-            segments[0].pos_marker.working_loc,
-            segments[0].get_type(),
-            len(segments),
-        )
-
-        best_match_length = 0
-        best_match: Optional[Tuple[MatchResult, MatchableType]] = None
-        # iterate at this position across all the matchers
-        for idx, matcher in enumerate(available_options):
-            # Check parse cache.
-            matcher_key = matcher.cache_key()
-            res_match: Optional[MatchResult] = parse_context.check_parse_cache(
-                loc_key, matcher_key
-            )
-            if res_match:
-                parse_match_logging(
-                    cls.__name__,
-                    "_look_ahead_match",
-                    "HIT",
-                    parse_context=parse_context,
-                    cache_hit=matcher.__class__.__name__,
-                    cache_key=matcher_key,
-                )
-            else:
-                # Match fresh if no cache hit
-                res_match = matcher.match(segments, parse_context)
-                # Cache it for later to for performance.
-                parse_context.put_parse_cache(loc_key, matcher_key, res_match)
-
-            if res_match.is_complete():
-                # Just return it! (WITH THE RIGHT OTHER STUFF)
-                parse_context.increment("complete_match")
-                if trim_noncode:
-                    return (
-                        MatchResult.from_matched(
-                            pre_nc + res_match.matched_segments + post_nc
-                        ),
-                        matcher,
-                    )
-                else:
-                    return res_match, matcher
-            elif res_match:
-                # We've got an incomplete match, if it's the best so far keep it.
-                if res_match.trimmed_matched_length > best_match_length:
-                    best_match = res_match, matcher
-                    best_match_length = res_match.trimmed_matched_length
-
-                    # If we've got a terminator next, it's an opportunity to
-                    # end earlier, and claim an effectively "complete" match.
-                    # NOTE: This means that by specifying terminators, we can
-                    # significantly increase performance.
-                    if idx == len(available_options) - 1:
-                        # If it's the last option - no need to check terminators.
-                        # We're going to end anyway, so we can skip that step.
-                        terminated = True
-                        break
-                    elif terminators:
-                        _, segs, _ = trim_non_code_segments(
-                            best_match[0].unmatched_segments
-                        )
-                        for terminator in terminators:
-                            terminator_match: MatchResult = terminator.match(
-                                segs, parse_context
-                            )
-
-                            if terminator_match.matched_segments:
-                                terminated = True
-                                break
-
-            if terminated:
-                break
-
-            # We could stash segments here, but given we might have some successful
-            # matches here, we shouldn't, because they'll be mutated in the wrong way.
-            # Eventually there might be a performance gain from doing that sensibly
-            # here.
-
-        if terminated:
-            parse_context.increment("terminated_match")
-        else:
-            parse_context.increment("unterminated_match")
-
-        # If we get here, then there wasn't a complete match. If we
-        # has a best_match, return that.
-        if best_match_length > 0:
-            assert best_match
-            # If not terminated, keep track of what the next token would
-            # have been if we had been able to terminate using it.
-            if not terminated:
-                if best_match[0].unmatched_segments:
-                    for seg in best_match[0].unmatched_segments:
-                        if seg.is_code:
-                            break
-                    next_seg = seg.raw_segments[0].raw_upper
-                else:  # pragma: no cover
-                    # NOTE: I don't think this clause should ever
-                    # occur, but it's included so that if it does happen
-                    # we don't get an exception and can better debug.
-                    next_seg = "<NONE>"
-                parse_context.parse_stats["next_counts"][next_seg] += 1
-
-            if trim_noncode:
-                return (
-                    MatchResult(
-                        pre_nc + best_match[0].matched_segments,
-                        best_match[0].unmatched_segments + post_nc,
-                    ),
-                    best_match[1],
-                )
-            else:
-                return best_match
-        # If no match at all, return nothing
-        return MatchResult.from_unmatched(segments), None
-
     def __str__(self) -> str:  # pragma: no cover TODO?
+        """Return a string representation of the object."""
         return repr(self)
 
     def __repr__(self) -> str:
+        """Return a string representation suitable for debugging."""
         return "<{}: [{}]>".format(
             self.__class__.__name__,
             curtail_string(
@@ -497,11 +217,11 @@ class BaseGrammar(Matchable):
 
     def copy(
         self: T,
-        insert: Optional[List[MatchableType]] = None,
+        insert: Optional[List[Matchable]] = None,
         at: Optional[int] = None,
-        before: Optional[Any] = None,
-        remove: Optional[List[MatchableType]] = None,
-        terminators: List[Union[str, MatchableType]] = [],
+        before: Optional[Matchable] = None,
+        remove: Optional[List[Matchable]] = None,
+        terminators: List[Union[str, Matchable]] = [],
         replace_terminators: bool = False,
         # NOTE: Optionally allow other kwargs to be provided to this
         # method for type compatibility. Any provided won't be used.
@@ -537,6 +257,9 @@ class BaseGrammar(Matchable):
             replace_terminators (:obj:`bool`, default False): When `True`
                 we replace the existing terminators from the copied grammar,
                 otherwise we just append.
+            **kwargs: Optional additional values may be passed to this
+                method for inherited classes, but if unused they will raise
+                an `AssertionError`.
         """
         assert not kwargs, f"Unexpected kwargs to .copy(): {kwargs}"
         # Copy only the *grammar* elements. The rest comes through
@@ -600,12 +323,11 @@ class Ref(BaseGrammar):
     def __init__(
         self,
         *args: str,
-        exclude: Optional[MatchableType] = None,
-        terminators: Sequence[Union[MatchableType, str]] = (),
+        exclude: Optional[Matchable] = None,
+        terminators: Sequence[Union[Matchable, str]] = (),
         reset_terminators: bool = False,
         allow_gaps: bool = True,
         optional: bool = False,
-        ephemeral_name: Optional[str] = None,
     ) -> None:
         # For Ref, there should only be one arg.
         assert len(args) == 1, (
@@ -620,7 +342,6 @@ class Ref(BaseGrammar):
             # NOTE: Don't pass on any args (we've already handled it with self._ref)
             allow_gaps=allow_gaps,
             optional=optional,
-            ephemeral_name=ephemeral_name,
             # Terminators don't take effect directly within this grammar, but
             # the Ref grammar is an effective place to manage the terminators
             # inherited via the context.
@@ -644,7 +365,7 @@ class Ref(BaseGrammar):
             crumbs=(crumbs or ()) + (self._ref,),
         )
 
-    def _get_elem(self, dialect: "Dialect") -> MatchableType:
+    def _get_elem(self, dialect: "Dialect") -> Matchable:
         """Get the actual object we're referencing."""
         if dialect:
             # Use the dialect to retrieve the grammar it refers to.
@@ -653,20 +374,31 @@ class Ref(BaseGrammar):
             raise ReferenceError("No Dialect has been provided to Ref grammar!")
 
     def __repr__(self) -> str:
+        """Return a string representation of the 'Ref' object."""
         return "<Ref: {}{}>".format(
             repr(self._ref), " [opt]" if self.is_optional() else ""
         )
 
-    @match_wrapper(v_level=4)  # Log less for Ref
-    @allow_ephemeral
     def match(
-        self, segments: Tuple[BaseSegment, ...], parse_context: ParseContext
-    ) -> "MatchResult":
+        self,
+        segments: Sequence["BaseSegment"],
+        idx: int,
+        parse_context: "ParseContext",
+    ) -> MatchResult:
         """Match a list of segments against this segment.
 
         Matching can be done from either the raw or the segments.
         This raw function can be overridden, or a grammar defined
         on the underlying class.
+
+        Args:
+            segments (Tuple[BaseSegment, ...]): The sequence of segments
+                to match against.
+            idx (int): Index of the element in the sequence.
+            parse_context (ParseContext): The parse context.
+
+        Returns:
+            MatchResult: The result of the matching process.
         """
         elem = self._get_elem(dialect=parse_context.dialect)
 
@@ -678,8 +410,8 @@ class Ref(BaseGrammar):
                 clear_terminators=self.reset_terminators,
                 push_terminators=self.terminators,
             ) as ctx:
-                if self.exclude.match(segments, ctx):
-                    return MatchResult.from_unmatched(segments)
+                if self.exclude.match(segments, idx, ctx):
+                    return MatchResult.empty_at(idx)
 
         # Match against that. NB We're not incrementing the match_depth here.
         # References shouldn't really count as a depth of match.
@@ -688,9 +420,7 @@ class Ref(BaseGrammar):
             clear_terminators=self.reset_terminators,
             push_terminators=self.terminators,
         ) as ctx:
-            resp = elem.match(segments, ctx)
-
-        return resp
+            return elem.match(segments, idx, parse_context)
 
     @classmethod
     def keyword(cls, keyword: str, optional: bool = False) -> BaseGrammar:
@@ -701,6 +431,13 @@ class Ref(BaseGrammar):
 
         Ref.keyword('select') == Ref('SelectKeywordSegment')
 
+        Args:
+            keyword (str): The name of the keyword.
+            optional (bool, optional): Whether the keyword is optional or
+                not. Defaults to False.
+
+        Returns:
+            BaseGrammar: An instance of the BaseGrammar class.
         """
         name = keyword.capitalize() + "KeywordSegment"
         return cls(name, optional=optional)
@@ -710,14 +447,38 @@ class Anything(BaseGrammar):
     """Matches anything."""
 
     def match(
-        self, segments: Tuple[BaseSegment, ...], parse_context: ParseContext
-    ) -> "MatchResult":
+        self,
+        segments: Sequence["BaseSegment"],
+        idx: int,
+        parse_context: "ParseContext",
+    ) -> MatchResult:
         """Matches... Anything.
 
         Most useful in match grammars, where a later parse grammar
         will work out what's inside.
+
+        NOTE: This grammar does still only match as far as any inherited
+        terminators if they exist.
         """
-        return MatchResult.from_matched(segments)
+        terminators = [*self.terminators]
+        if not self.reset_terminators:
+            # Only add context terminators if we're not resetting.
+            terminators.extend(parse_context.terminators)
+        if not terminators:
+            return MatchResult(slice(idx, len(segments)))
+
+        return greedy_match(
+            segments,
+            idx,
+            parse_context,
+            terminators,
+            # Using the nested match option means that we can match
+            # any bracketed sections we find to persist the structure
+            # even if this grammar is permissive on the meaning.
+            # This preserves backward compatibility with older
+            # parsing behaviour.
+            nested_match=True,
+        )
 
 
 class Nothing(BaseGrammar):
@@ -728,11 +489,10 @@ class Nothing(BaseGrammar):
     """
 
     def match(
-        self, segments: Tuple[BaseSegment, ...], parse_context: ParseContext
-    ) -> "MatchResult":
-        """Matches... nothing.
-
-        Useful for placeholders which might be overwritten by other
-        dialects.
-        """
-        return MatchResult.from_unmatched(segments)
+        self,
+        segments: Sequence["BaseSegment"],
+        idx: int,
+        parse_context: "ParseContext",
+    ) -> MatchResult:
+        """Always return a failed (empty) match."""
+        return MatchResult.empty_at(idx)
