@@ -159,16 +159,40 @@ class Rule_LT09(BaseRule):
         self, select_targets_info, segment
     ) -> Optional[LintResult]:
         """Multiple select targets. Ensure each is on a separate line."""
-        # Insert newline before every select target.
         fixes = []
+        previous_code = None
+        select_clause_raws = Segments(segment).raw_segments
         for i, select_target in enumerate(select_targets_info.select_targets):
-            base_segment = (
-                segment if not i else select_targets_info.select_targets[i - 1]
+            assert select_target.pos_marker
+            target_start_line = select_target.pos_marker.working_line_no
+            target_initial_code = (
+                Segments(select_target).raw_segments.first(sp.is_code()).get()
             )
-            if (
-                base_segment.pos_marker.working_line_no
-                == select_target.pos_marker.working_line_no
-            ):
+            assert target_initial_code
+            previous_code = (
+                select_clause_raws.select(
+                    # Get the first code that isn't a comma.
+                    select_if=sp.and_(sp.is_code(), sp.not_(sp.raw_is(","))),
+                    start_seg=previous_code,
+                    stop_seg=target_initial_code,
+                )
+                .last()
+                .get()
+            )
+            assert previous_code
+            assert previous_code.pos_marker
+            previous_end_line = previous_code.pos_marker.working_line_no
+            self.logger.debug(
+                "- Evaluating %s [%s, %s]: Prev ends with: %s",
+                select_target,
+                previous_end_line,
+                target_start_line,
+                previous_code,
+            )
+
+            # Check whether this target *starts* on the same line that the
+            # previous one *ends* on. If they are on the same line, insert a newline.
+            if target_start_line == previous_end_line:
                 # Find and delete any whitespace before the select target.
                 start_seg = select_targets_info.select_idx
                 # If any select modifier (e.g. distinct ) is present, start
@@ -178,9 +202,11 @@ class Rule_LT09(BaseRule):
                     start_seg = segment.segments.index(modifier)
 
                 ws_to_delete = segment.select_children(
-                    start_seg=segment.segments[start_seg]
-                    if not i
-                    else select_targets_info.select_targets[i - 1],
+                    start_seg=(
+                        segment.segments[start_seg]
+                        if not i
+                        else select_targets_info.select_targets[i - 1]
+                    ),
                     select_if=lambda s: s.is_type("whitespace"),
                     loop_while=lambda s: s.is_type("whitespace", "comma") or s.is_meta,
                 )
@@ -217,65 +243,65 @@ class Rule_LT09(BaseRule):
     ):
         select_clause = FunctionalContext(context).segment
         parent_stack = context.parent_stack
+        target_idx = select_targets_info.first_select_target_idx
+        select_children = select_clause.children()
+        target_seg = select_children[target_idx]
 
         # If it's all on one line, then there's no issue.
         if not (
             select_targets_info.select_idx
             < select_targets_info.first_new_line_idx
-            < select_targets_info.first_select_target_idx
+            < target_idx
         ):
             self.logger.info(
                 "Target at index %s is already on a single line.",
-                select_targets_info.first_select_target_idx,
+                target_idx,
             )
             return None
 
-        # Do we have a modifier?
-        select_children = select_clause.children()
-        modifier: Optional[Segments]
-        modifier = select_children.first(sp.is_type("select_clause_modifier"))
-
         # Does the target contain a newline?
         # i.e. even if it's a single element, does it already span more than
         # one line?
-        if (
-            "newline"
-            in select_children[
-                select_targets_info.first_select_target_idx
-            ].descendant_type_set
-        ):
+        if "newline" in target_seg.descendant_type_set:
             self.logger.info(
                 "Target at index %s spans multiple lines so ignoring.",
-                select_targets_info.first_select_target_idx,
+                target_idx,
             )
             return None
 
+        if select_targets_info.comment_after_select_idx != -1:
+            # The SELECT is followed by a comment on the same line. In order
+            # to autofix this, we'd need to move the select target between
+            # SELECT and the comment and potentially delete the entire line
+            # where the select target was (if it is now empty). This is
+            # *fairly tricky and complex*, in part because the newline on
+            # the select target's line is several levels higher in the
+            # parser tree. Hence, we currently don't autofix this. Could be
+            # autofixed in the future if/when we have the time.
+            return LintResult(anchor=select_clause.get())
+
         # Prepare the select clause which will be inserted
-        insert_buff = [
-            WhitespaceSegment(),
-            select_children[select_targets_info.first_select_target_idx],
-        ]
+        insert_buff = [WhitespaceSegment(), target_seg]
+        # Delete the first select target from its original location.
+        # We'll add it to the right section at the end, once we know
+        # what to add.
+        initial_deletes = [target_seg]
+        # If there's whitespace before it, delete that too.
+        if select_children[target_idx - 1].is_type("whitespace"):
+            initial_deletes.append(select_children[target_idx - 1])
 
-        # Check if the modifier is one we care about
-        if modifier:
-            # If it's already on the first line, ignore it.
-            if (
-                select_children.index(modifier.get())
-                < select_targets_info.first_new_line_idx
-            ):
-                modifier = None
-        fixes = [
-            # Delete the first select target from its original location.
-            # We'll add it to the right section at the end, once we know
-            # what to add.
-            LintFix.delete(
-                select_children[select_targets_info.first_select_target_idx],
-            ),
-        ]
+        # Do we have a modifier?
+        modifier: Optional[Segments]
+        modifier = select_children.first(sp.is_type("select_clause_modifier"))
 
-        # If we have a modifier to move:
-        if modifier:
-            # Add it to the insert
+        if (
+            # Check if the modifier is one we care about
+            modifier
+            # We only care if it's not already on the first line.
+            and select_children.index(modifier.get())
+            >= select_targets_info.first_new_line_idx
+        ):
+            # Prepend it to the insert buffer
             insert_buff = [WhitespaceSegment(), modifier[0]] + insert_buff
 
             modifier_idx = select_children.index(modifier.get())
@@ -284,81 +310,43 @@ class Rule_LT09(BaseRule):
                 len(select_children) > modifier_idx + 1
                 and select_children[modifier_idx + 2].is_whitespace
             ):
-                fixes += [
-                    LintFix.delete(
-                        select_children[modifier_idx + 2],
-                    ),
-                ]
+                initial_deletes.append(select_children[modifier_idx + 2])
 
             # Delete the modifier itself
-            fixes += [
-                LintFix.delete(
-                    modifier[0],
-                ),
-            ]
+            initial_deletes.append(modifier[0])
 
             # Set the position marker for removing the preceding
             # whitespace and newline, which we'll use below.
             start_idx = modifier_idx
+            start_seg = modifier[0]
         else:
             # Set the position marker for removing the preceding
             # whitespace and newline, which we'll use below.
-            start_idx = select_targets_info.first_select_target_idx
+            start_idx = target_idx
+            start_seg = select_children[select_targets_info.first_new_line_idx]
+
+        fixes = [
+            # Insert the select_clause in place of the first newline in the
+            # Select statement
+            LintFix.replace(
+                select_children[select_targets_info.first_new_line_idx],
+                insert_buff,
+            ),
+            # Materialise any deletes so far...
+            *(LintFix.delete(seg) for seg in initial_deletes),
+        ]
 
         if parent_stack and parent_stack[-1].is_type("select_statement"):
             select_stmt = parent_stack[-1]
             select_clause_idx = select_stmt.segments.index(select_clause.get())
             after_select_clause_idx = select_clause_idx + 1
+
             if len(select_stmt.segments) > after_select_clause_idx:
+                add_newline = True
+                to_delete: Sequence[BaseSegment] = [target_seg]
+                next_segment = select_stmt.segments[after_select_clause_idx]
 
-                def _fixes_for_move_after_select_clause(
-                    stop_seg: BaseSegment,
-                    delete_segments: Optional[Segments] = None,
-                    add_newline: bool = True,
-                ) -> List[LintFix]:
-                    """Cleans up by moving leftover select_clause segments.
-
-                    Context: Some of the other fixes we make in
-                    _eval_single_select_target_element() leave leftover
-                    child segments that need to be moved to become
-                    *siblings* of the select_clause.
-                    """
-                    start_seg = (
-                        modifier[0]
-                        if modifier
-                        else select_children[select_targets_info.first_new_line_idx]
-                    )
-                    move_after_select_clause = select_children.select(
-                        start_seg=start_seg,
-                        stop_seg=stop_seg,
-                    )
-                    # :TRICKY: Below, we have a couple places where we
-                    # filter to guard against deleting the same segment
-                    # multiple times -- this is illegal.
-                    all_deletes = set(
-                        fix.anchor for fix in fixes if fix.edit_type == "delete"
-                    )
-                    fixes_ = []
-                    for seg in delete_segments or []:
-                        if seg not in all_deletes:
-                            fixes.append(LintFix.delete(seg))
-                            all_deletes.add(seg)
-                    fixes_ += [
-                        LintFix.delete(seg)
-                        for seg in move_after_select_clause
-                        if seg not in all_deletes
-                    ]
-                    if move_after_select_clause or add_newline:
-                        fixes_.append(
-                            LintFix.create_after(
-                                select_clause[0],
-                                ([NewlineSegment()] if add_newline else [])
-                                + list(move_after_select_clause),
-                            )
-                        )
-                    return fixes_
-
-                if select_stmt.segments[after_select_clause_idx].is_type("newline"):
+                if next_segment.is_type("newline"):
                     # Since we're deleting the newline, we should also delete all
                     # whitespace before it or it will add random whitespace to
                     # following statements. So walk back through the segment
@@ -379,69 +367,46 @@ class Rule_LT09(BaseRule):
 
                         # Delete the newline if we decided to.
                         if delete_last_newline:
-                            fixes.append(
-                                LintFix.delete(
-                                    select_stmt.segments[after_select_clause_idx],
-                                )
-                            )
+                            fixes.append(LintFix.delete(next_segment))
 
-                        fixes += _fixes_for_move_after_select_clause(
-                            to_delete[-1], to_delete
-                        )
-                elif select_stmt.segments[after_select_clause_idx].is_type(
-                    "whitespace"
-                ):
+                elif next_segment.is_type("whitespace"):
                     # The select_clause has stuff after (most likely a comment)
                     # Delete the whitespace immediately after the select clause
                     # so the other stuff aligns nicely based on where the select
                     # clause started.
-                    fixes += [
-                        LintFix.delete(
-                            select_stmt.segments[after_select_clause_idx],
-                        ),
-                    ]
-                    fixes += _fixes_for_move_after_select_clause(
-                        select_children[select_targets_info.first_select_target_idx],
+                    fixes.append(LintFix.delete(next_segment))
+
+                if to_delete:
+                    # Clean up by moving leftover select_clause segments.
+
+                    # Context: Some of the other fixes we make in
+                    # _eval_single_select_target_element() leave leftover
+                    # child segments that need to be moved to become
+                    # *siblings* of the select_clause.
+                    move_after_select_clause = select_children.select(
+                        start_seg=start_seg,
+                        stop_seg=to_delete[-1],
                     )
-                elif select_stmt.segments[after_select_clause_idx].is_type("dedent"):
-                    # Again let's strip back the whitespace, but simpler
-                    # as don't need to worry about new line so just break
-                    # if see non-whitespace
-                    to_delete = select_children.reversed().select(
-                        loop_while=sp.is_type("whitespace"),
-                        start_seg=select_children[select_clause_idx - 1],
+                    # :TRICKY: Below, we have a couple places where we
+                    # filter to guard against deleting the same segment
+                    # multiple times -- this is illegal.
+                    all_deletes = set(
+                        fix.anchor for fix in fixes if fix.edit_type == "delete"
                     )
-                    if to_delete:
-                        fixes += _fixes_for_move_after_select_clause(
-                            to_delete[-1],
-                            to_delete,
-                            # If we deleted a newline, create a newline.
-                            any(seg for seg in to_delete if seg.is_type("newline")),
+                    for seg in (*to_delete, *move_after_select_clause):
+                        if seg not in all_deletes:
+                            fixes.append(LintFix.delete(seg))
+                            all_deletes.add(seg)
+
+                    if move_after_select_clause or add_newline:
+                        fixes.append(
+                            LintFix.create_after(
+                                select_clause[0],
+                                ([NewlineSegment()] if add_newline else [])
+                                + list(move_after_select_clause),
+                            )
                         )
-                else:
-                    fixes += _fixes_for_move_after_select_clause(
-                        select_children[select_targets_info.first_select_target_idx],
-                    )
 
-        if select_targets_info.comment_after_select_idx == -1:
-            fixes += [
-                # Insert the select_clause in place of the first newline in the
-                # Select statement
-                LintFix.replace(
-                    select_children[select_targets_info.first_new_line_idx],
-                    insert_buff,
-                ),
-            ]
-        else:
-            # The SELECT is followed by a comment on the same line. In order
-            # to autofix this, we'd need to move the select target between
-            # SELECT and the comment and potentially delete the entire line
-            # where the select target was (if it is now empty). This is
-            # *fairly tricky and complex*, in part because the newline on
-            # the select target's line is several levels higher in the
-            # parser tree. Hence, we currently don't autofix this. Could be
-            # autofixed in the future if/when we have the time.
-            fixes = []
         return LintResult(
             anchor=select_clause.get(),
             fixes=fixes,
